import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.ae_assessment_action import AeAssessmentAction
from ..models.ae_employee_state import AeEmployeeState
from ..models.ae_exclusion_code import AeExclusionCode
from ..models.ae_status import AeStatus
from ..models.ae_uk_worker import AeUKWorker
from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..types import UNSET, Unset

T = TypeVar("T", bound="AeAssessment")


@attr.s(auto_attribs=True)
class AeAssessment:
    """As part of AutoEnrolment we assess your Employees to see if they need to be auto-enroled in a Pension.
    This model shows the result of an assessment.

        Attributes:
            assessment_date (Union[Unset, datetime.datetime]):
            employee_state (Union[Unset, AeEmployeeState]):
            age (Union[Unset, int]): [readonly] The age of the Employee at the time of the assessment
            uk_worker (Union[Unset, AeUKWorker]):
            pay_period (Union[Unset, PayPeriods]):
            ordinal (Union[Unset, int]): [readonly] The PaySchedule ordinal for the Employee at the time of the assessment
            earnings_in_period (Union[Unset, float]): [readonly]
            qualifying_earnings_in_period (Union[Unset, float]): [readonly]
            ae_exclusion_code (Union[Unset, AeExclusionCode]):
            status (Union[Unset, AeStatus]):
            reason (Union[Unset, None, str]): [readonly] The reason for determining the Status given.
            action (Union[Unset, AeAssessmentAction]):
            employee (Union[Unset, Item]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    assessment_date: Union[Unset, datetime.datetime] = UNSET
    employee_state: Union[Unset, AeEmployeeState] = UNSET
    age: Union[Unset, int] = UNSET
    uk_worker: Union[Unset, AeUKWorker] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    earnings_in_period: Union[Unset, float] = UNSET
    qualifying_earnings_in_period: Union[Unset, float] = UNSET
    ae_exclusion_code: Union[Unset, AeExclusionCode] = UNSET
    status: Union[Unset, AeStatus] = UNSET
    reason: Union[Unset, None, str] = UNSET
    action: Union[Unset, AeAssessmentAction] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        assessment_date: Union[Unset, str] = UNSET
        if not isinstance(self.assessment_date, Unset):
            assessment_date = self.assessment_date.isoformat()

        employee_state: Union[Unset, str] = UNSET
        if not isinstance(self.employee_state, Unset):
            employee_state = self.employee_state.value

        age = self.age
        uk_worker: Union[Unset, str] = UNSET
        if not isinstance(self.uk_worker, Unset):
            uk_worker = self.uk_worker.value

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        earnings_in_period = self.earnings_in_period
        qualifying_earnings_in_period = self.qualifying_earnings_in_period
        ae_exclusion_code: Union[Unset, str] = UNSET
        if not isinstance(self.ae_exclusion_code, Unset):
            ae_exclusion_code = self.ae_exclusion_code.value

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        reason = self.reason
        action: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.to_dict()

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if assessment_date is not UNSET:
            field_dict["assessmentDate"] = assessment_date
        if employee_state is not UNSET:
            field_dict["employeeState"] = employee_state
        if age is not UNSET:
            field_dict["age"] = age
        if uk_worker is not UNSET:
            field_dict["ukWorker"] = uk_worker
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if earnings_in_period is not UNSET:
            field_dict["earningsInPeriod"] = earnings_in_period
        if qualifying_earnings_in_period is not UNSET:
            field_dict["qualifyingEarningsInPeriod"] = qualifying_earnings_in_period
        if ae_exclusion_code is not UNSET:
            field_dict["aeExclusionCode"] = ae_exclusion_code
        if status is not UNSET:
            field_dict["status"] = status
        if reason is not UNSET:
            field_dict["reason"] = reason
        if action is not UNSET:
            field_dict["action"] = action
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _assessment_date = d.pop("assessmentDate", UNSET)
        assessment_date: Union[Unset, datetime.datetime]
        if isinstance(_assessment_date, Unset):
            assessment_date = UNSET
        else:
            assessment_date = isoparse(_assessment_date)

        _employee_state = d.pop("employeeState", UNSET)
        employee_state: Union[Unset, AeEmployeeState]
        if isinstance(_employee_state, Unset):
            employee_state = UNSET
        else:
            employee_state = AeEmployeeState(_employee_state)

        age = d.pop("age", UNSET)

        _uk_worker = d.pop("ukWorker", UNSET)
        uk_worker: Union[Unset, AeUKWorker]
        if isinstance(_uk_worker, Unset):
            uk_worker = UNSET
        else:
            uk_worker = AeUKWorker(_uk_worker)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        earnings_in_period = d.pop("earningsInPeriod", UNSET)

        qualifying_earnings_in_period = d.pop("qualifyingEarningsInPeriod", UNSET)

        _ae_exclusion_code = d.pop("aeExclusionCode", UNSET)
        ae_exclusion_code: Union[Unset, AeExclusionCode]
        if isinstance(_ae_exclusion_code, Unset):
            ae_exclusion_code = UNSET
        else:
            ae_exclusion_code = AeExclusionCode(_ae_exclusion_code)

        _status = d.pop("status", UNSET)
        status: Union[Unset, AeStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = AeStatus(_status)

        reason = d.pop("reason", UNSET)

        _action = d.pop("action", UNSET)
        action: Union[Unset, AeAssessmentAction]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = AeAssessmentAction.from_dict(_action)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        ae_assessment = cls(
            assessment_date=assessment_date,
            employee_state=employee_state,
            age=age,
            uk_worker=uk_worker,
            pay_period=pay_period,
            ordinal=ordinal,
            earnings_in_period=earnings_in_period,
            qualifying_earnings_in_period=qualifying_earnings_in_period,
            ae_exclusion_code=ae_exclusion_code,
            status=status,
            reason=reason,
            action=action,
            employee=employee,
            id=id,
        )

        return ae_assessment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BureauSettings")


@attr.s(auto_attribs=True)
class BureauSettings:
    """Represents the BureauSettings for an Employer.

    Attributes:
        enable_approvals (Union[Unset, bool]): Whether or not Payruns for this employer need to go through an Approval
            process
        processor_user_id (Union[Unset, None, str]): The Id of the user, if any, that is the allocated Processor
        report_pack_id (Union[Unset, None, str]): The Id of the ReportPack, if any, to present to the Payroll Client
        is_bacs_client (Union[Unset, bool]): Indicates that the bureau is responsible for BACS payments of net wages to
            employees
        is_bacs_client_for_hmrc (Union[Unset, bool]): Indicates that the bureau is responsible for BACS payments of HMRC
            liabilites
        show_contact_card (Union[Unset, bool]): Show a Contact card with the Processors details  on the PayrollClient
            dashboard
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    enable_approvals: Union[Unset, bool] = UNSET
    processor_user_id: Union[Unset, None, str] = UNSET
    report_pack_id: Union[Unset, None, str] = UNSET
    is_bacs_client: Union[Unset, bool] = UNSET
    is_bacs_client_for_hmrc: Union[Unset, bool] = UNSET
    show_contact_card: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        enable_approvals = self.enable_approvals
        processor_user_id = self.processor_user_id
        report_pack_id = self.report_pack_id
        is_bacs_client = self.is_bacs_client
        is_bacs_client_for_hmrc = self.is_bacs_client_for_hmrc
        show_contact_card = self.show_contact_card
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if enable_approvals is not UNSET:
            field_dict["enableApprovals"] = enable_approvals
        if processor_user_id is not UNSET:
            field_dict["processorUserId"] = processor_user_id
        if report_pack_id is not UNSET:
            field_dict["reportPackId"] = report_pack_id
        if is_bacs_client is not UNSET:
            field_dict["isBacsClient"] = is_bacs_client
        if is_bacs_client_for_hmrc is not UNSET:
            field_dict["isBacsClientForHmrc"] = is_bacs_client_for_hmrc
        if show_contact_card is not UNSET:
            field_dict["showContactCard"] = show_contact_card
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enable_approvals = d.pop("enableApprovals", UNSET)

        processor_user_id = d.pop("processorUserId", UNSET)

        report_pack_id = d.pop("reportPackId", UNSET)

        is_bacs_client = d.pop("isBacsClient", UNSET)

        is_bacs_client_for_hmrc = d.pop("isBacsClientForHmrc", UNSET)

        show_contact_card = d.pop("showContactCard", UNSET)

        id = d.pop("id", UNSET)

        bureau_settings = cls(
            enable_approvals=enable_approvals,
            processor_user_id=processor_user_id,
            report_pack_id=report_pack_id,
            is_bacs_client=is_bacs_client,
            is_bacs_client_for_hmrc=is_bacs_client_for_hmrc,
            show_contact_card=show_contact_card,
            id=id,
        )

        return bureau_settings



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.opening_balances import OpeningBalances
from ...types import Response


def _get_kwargs(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{employeeId}/openingBalances/p45".format(
        client.base_url, employerId=employer_id, employeeId=employee_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, OpeningBalances]]:
    if response.status_code == 200:
        response_200 = OpeningBalances.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, OpeningBalances]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Response[Union[Any, OpeningBalances]]:
    """Update P45 Value

     Updates the P45 Values on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the P45 values.
    There must be a currently open PayRun for the employee.
    Only the PreviousEmployerGross and PreviousEmployerTax properties of the submitted OpeningBalances
    model are updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Optional[Union[Any, OpeningBalances]]:
    """Update P45 Value

     Updates the P45 Values on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the P45 values.
    There must be a currently open PayRun for the employee.
    Only the PreviousEmployerGross and PreviousEmployerTax properties of the submitted OpeningBalances
    model are updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    return sync_detailed(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Response[Union[Any, OpeningBalances]]:
    """Update P45 Value

     Updates the P45 Values on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the P45 values.
    There must be a currently open PayRun for the employee.
    Only the PreviousEmployerGross and PreviousEmployerTax properties of the submitted OpeningBalances
    model are updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Optional[Union[Any, OpeningBalances]]:
    """Update P45 Value

     Updates the P45 Values on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the P45 values.
    There must be a currently open PayRun for the employee.
    Only the PreviousEmployerGross and PreviousEmployerTax properties of the submitted OpeningBalances
    model are updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            employee_id=employee_id,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.fps_employee_details import FpsEmployeeDetails
from ..models.fps_employment import FpsEmployment
from ..models.item import Item
from ..models.rti_validation_warning import RtiValidationWarning
from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployee")


@attr.s(auto_attribs=True)
class FpsEmployee:
    """
    Attributes:
        employee_unique_id (Union[Unset, str]):
        item (Union[Unset, Item]):
        employee_details (Union[Unset, FpsEmployeeDetails]):
        employment (Union[Unset, FpsEmployment]):
        validation_warnings (Union[Unset, None, List[RtiValidationWarning]]):
    """

    employee_unique_id: Union[Unset, str] = UNSET
    item: Union[Unset, Item] = UNSET
    employee_details: Union[Unset, FpsEmployeeDetails] = UNSET
    employment: Union[Unset, FpsEmployment] = UNSET
    validation_warnings: Union[Unset, None, List[RtiValidationWarning]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_unique_id = self.employee_unique_id
        item: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.item, Unset):
            item = self.item.to_dict()

        employee_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee_details, Unset):
            employee_details = self.employee_details.to_dict()

        employment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employment, Unset):
            employment = self.employment.to_dict()

        validation_warnings: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.validation_warnings, Unset):
            if self.validation_warnings is None:
                validation_warnings = None
            else:
                validation_warnings = []
                for validation_warnings_item_data in self.validation_warnings:
                    validation_warnings_item = validation_warnings_item_data.to_dict()

                    validation_warnings.append(validation_warnings_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_unique_id is not UNSET:
            field_dict["employeeUniqueId"] = employee_unique_id
        if item is not UNSET:
            field_dict["item"] = item
        if employee_details is not UNSET:
            field_dict["employeeDetails"] = employee_details
        if employment is not UNSET:
            field_dict["employment"] = employment
        if validation_warnings is not UNSET:
            field_dict["validationWarnings"] = validation_warnings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_unique_id = d.pop("employeeUniqueId", UNSET)

        _item = d.pop("item", UNSET)
        item: Union[Unset, Item]
        if isinstance(_item, Unset):
            item = UNSET
        else:
            item = Item.from_dict(_item)

        _employee_details = d.pop("employeeDetails", UNSET)
        employee_details: Union[Unset, FpsEmployeeDetails]
        if isinstance(_employee_details, Unset):
            employee_details = UNSET
        else:
            employee_details = FpsEmployeeDetails.from_dict(_employee_details)

        _employment = d.pop("employment", UNSET)
        employment: Union[Unset, FpsEmployment]
        if isinstance(_employment, Unset):
            employment = UNSET
        else:
            employment = FpsEmployment.from_dict(_employment)

        validation_warnings = []
        _validation_warnings = d.pop("validationWarnings", UNSET)
        for validation_warnings_item_data in _validation_warnings or []:
            validation_warnings_item = RtiValidationWarning.from_dict(
                validation_warnings_item_data
            )

            validation_warnings.append(validation_warnings_item)

        fps_employee = cls(
            employee_unique_id=employee_unique_id,
            item=item,
            employee_details=employee_details,
            employment=employment,
            validation_warnings=validation_warnings,
        )

        return fps_employee



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployeePay")


@attr.s(auto_attribs=True)
class PapdisEmployeePay:
    """
    Attributes:
        pensionable_earnings_amount (Union[Unset, float]): [readonly]
        total_gross_qualifying_earnings_amount (Union[Unset, float]): [readonly]
    """

    pensionable_earnings_amount: Union[Unset, float] = UNSET
    total_gross_qualifying_earnings_amount: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pensionable_earnings_amount = self.pensionable_earnings_amount
        total_gross_qualifying_earnings_amount = (
            self.total_gross_qualifying_earnings_amount
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pensionable_earnings_amount is not UNSET:
            field_dict["pensionableEarningsAmount"] = pensionable_earnings_amount
        if total_gross_qualifying_earnings_amount is not UNSET:
            field_dict[
                "totalGrossQualifyingEarningsAmount"
            ] = total_gross_qualifying_earnings_amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pensionable_earnings_amount = d.pop("pensionableEarningsAmount", UNSET)

        total_gross_qualifying_earnings_amount = d.pop(
            "totalGrossQualifyingEarningsAmount", UNSET
        )

        papdis_employee_pay = cls(
            pensionable_earnings_amount=pensionable_earnings_amount,
            total_gross_qualifying_earnings_amount=total_gross_qualifying_earnings_amount,
        )

        return papdis_employee_pay



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.bank_details import BankDetails
from ..models.external_data_company import ExternalDataCompany
from ..models.external_data_provider_id import ExternalDataProviderId
from ..models.pay_method import PayMethod
from ..models.pension_administrator import PensionAdministrator
from ..models.pension_provider import PensionProvider
from ..models.pension_rule import PensionRule
from ..models.worker_group import WorkerGroup
from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionScheme")


@attr.s(auto_attribs=True)
class PensionScheme:
    """
    Attributes:
        name (str):
        provider (Union[Unset, PensionProvider]):
        administrator (Union[Unset, PensionAdministrator]):
        pension_rule (Union[Unset, PensionRule]):
        qualifying_scheme (Union[Unset, bool]): Set to true if this is a Qualifying Scheme for Auto Enrolment
        disable_ae_letters (Union[Unset, bool]): Set to true if the provider deals with AutoEnrolment Letters and
            therefore the system should not generate them
        subtract_basic_rate_tax (Union[Unset, bool]):
        pay_method (Union[Unset, PayMethod]):
        bank_details (Union[Unset, BankDetails]):
        use_custom_pay_codes (Union[Unset, bool]): If set to true then rather than using the setting on the PayCode to
            determine if the pay is pensionable
            we'll instead treat it as pensionable if the Code is included in CustomPayCodes
        custom_pay_codes (Union[Unset, None, List[str]]): If UseCustomPayCodes is set to true then this contains a list
            of PayCodes.Code
            that we'll treat as being pensionable.
        worker_groups (Union[Unset, None, List[WorkerGroup]]): A list of WorkerGroups for this Pension. There must
            always be at least one WorkerGroup
        external_data_provider_id (Union[Unset, ExternalDataProviderId]):
        external_data_company (Union[Unset, ExternalDataCompany]): When we retrieve data from an ExternalDataProvider we
            normalise it so that regardless of the provider the models are the same.
            This model is used to represent a Company in an ExternalDataProvider
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    name: str
    provider: Union[Unset, PensionProvider] = UNSET
    administrator: Union[Unset, PensionAdministrator] = UNSET
    pension_rule: Union[Unset, PensionRule] = UNSET
    qualifying_scheme: Union[Unset, bool] = UNSET
    disable_ae_letters: Union[Unset, bool] = UNSET
    subtract_basic_rate_tax: Union[Unset, bool] = UNSET
    pay_method: Union[Unset, PayMethod] = UNSET
    bank_details: Union[Unset, BankDetails] = UNSET
    use_custom_pay_codes: Union[Unset, bool] = UNSET
    custom_pay_codes: Union[Unset, None, List[str]] = UNSET
    worker_groups: Union[Unset, None, List[WorkerGroup]] = UNSET
    external_data_provider_id: Union[Unset, ExternalDataProviderId] = UNSET
    external_data_company: Union[Unset, ExternalDataCompany] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        provider: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.provider, Unset):
            provider = self.provider.to_dict()

        administrator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.administrator, Unset):
            administrator = self.administrator.to_dict()

        pension_rule: Union[Unset, str] = UNSET
        if not isinstance(self.pension_rule, Unset):
            pension_rule = self.pension_rule.value

        qualifying_scheme = self.qualifying_scheme
        disable_ae_letters = self.disable_ae_letters
        subtract_basic_rate_tax = self.subtract_basic_rate_tax
        pay_method: Union[Unset, str] = UNSET
        if not isinstance(self.pay_method, Unset):
            pay_method = self.pay_method.value

        bank_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bank_details, Unset):
            bank_details = self.bank_details.to_dict()

        use_custom_pay_codes = self.use_custom_pay_codes
        custom_pay_codes: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.custom_pay_codes, Unset):
            if self.custom_pay_codes is None:
                custom_pay_codes = None
            else:
                custom_pay_codes = self.custom_pay_codes

        worker_groups: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.worker_groups, Unset):
            if self.worker_groups is None:
                worker_groups = None
            else:
                worker_groups = []
                for worker_groups_item_data in self.worker_groups:
                    worker_groups_item = worker_groups_item_data.to_dict()

                    worker_groups.append(worker_groups_item)

        external_data_provider_id: Union[Unset, str] = UNSET
        if not isinstance(self.external_data_provider_id, Unset):
            external_data_provider_id = self.external_data_provider_id.value

        external_data_company: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.external_data_company, Unset):
            external_data_company = self.external_data_company.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if provider is not UNSET:
            field_dict["provider"] = provider
        if administrator is not UNSET:
            field_dict["administrator"] = administrator
        if pension_rule is not UNSET:
            field_dict["pensionRule"] = pension_rule
        if qualifying_scheme is not UNSET:
            field_dict["qualifyingScheme"] = qualifying_scheme
        if disable_ae_letters is not UNSET:
            field_dict["disableAeLetters"] = disable_ae_letters
        if subtract_basic_rate_tax is not UNSET:
            field_dict["subtractBasicRateTax"] = subtract_basic_rate_tax
        if pay_method is not UNSET:
            field_dict["payMethod"] = pay_method
        if bank_details is not UNSET:
            field_dict["bankDetails"] = bank_details
        if use_custom_pay_codes is not UNSET:
            field_dict["useCustomPayCodes"] = use_custom_pay_codes
        if custom_pay_codes is not UNSET:
            field_dict["customPayCodes"] = custom_pay_codes
        if worker_groups is not UNSET:
            field_dict["workerGroups"] = worker_groups
        if external_data_provider_id is not UNSET:
            field_dict["externalDataProviderId"] = external_data_provider_id
        if external_data_company is not UNSET:
            field_dict["externalDataCompany"] = external_data_company
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        _provider = d.pop("provider", UNSET)
        provider: Union[Unset, PensionProvider]
        if isinstance(_provider, Unset):
            provider = UNSET
        else:
            provider = PensionProvider.from_dict(_provider)

        _administrator = d.pop("administrator", UNSET)
        administrator: Union[Unset, PensionAdministrator]
        if isinstance(_administrator, Unset):
            administrator = UNSET
        else:
            administrator = PensionAdministrator.from_dict(_administrator)

        _pension_rule = d.pop("pensionRule", UNSET)
        pension_rule: Union[Unset, PensionRule]
        if isinstance(_pension_rule, Unset):
            pension_rule = UNSET
        else:
            pension_rule = PensionRule(_pension_rule)

        qualifying_scheme = d.pop("qualifyingScheme", UNSET)

        disable_ae_letters = d.pop("disableAeLetters", UNSET)

        subtract_basic_rate_tax = d.pop("subtractBasicRateTax", UNSET)

        _pay_method = d.pop("payMethod", UNSET)
        pay_method: Union[Unset, PayMethod]
        if isinstance(_pay_method, Unset):
            pay_method = UNSET
        else:
            pay_method = PayMethod(_pay_method)

        _bank_details = d.pop("bankDetails", UNSET)
        bank_details: Union[Unset, BankDetails]
        if isinstance(_bank_details, Unset):
            bank_details = UNSET
        else:
            bank_details = BankDetails.from_dict(_bank_details)

        use_custom_pay_codes = d.pop("useCustomPayCodes", UNSET)

        custom_pay_codes = cast(List[str], d.pop("customPayCodes", UNSET))

        worker_groups = []
        _worker_groups = d.pop("workerGroups", UNSET)
        for worker_groups_item_data in _worker_groups or []:
            worker_groups_item = WorkerGroup.from_dict(worker_groups_item_data)

            worker_groups.append(worker_groups_item)

        _external_data_provider_id = d.pop("externalDataProviderId", UNSET)
        external_data_provider_id: Union[Unset, ExternalDataProviderId]
        if isinstance(_external_data_provider_id, Unset):
            external_data_provider_id = UNSET
        else:
            external_data_provider_id = ExternalDataProviderId(
                _external_data_provider_id
            )

        _external_data_company = d.pop("externalDataCompany", UNSET)
        external_data_company: Union[Unset, ExternalDataCompany]
        if isinstance(_external_data_company, Unset):
            external_data_company = UNSET
        else:
            external_data_company = ExternalDataCompany.from_dict(
                _external_data_company
            )

        id = d.pop("id", UNSET)

        pension_scheme = cls(
            name=name,
            provider=provider,
            administrator=administrator,
            pension_rule=pension_rule,
            qualifying_scheme=qualifying_scheme,
            disable_ae_letters=disable_ae_letters,
            subtract_basic_rate_tax=subtract_basic_rate_tax,
            pay_method=pay_method,
            bank_details=bank_details,
            use_custom_pay_codes=use_custom_pay_codes,
            custom_pay_codes=custom_pay_codes,
            worker_groups=worker_groups,
            external_data_provider_id=external_data_provider_id,
            external_data_company=external_data_company,
            id=id,
        )

        return pension_scheme



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..models.pay_run import PayRun
from ..models.recoverable_amounts import RecoverableAmounts
from ..models.report import Report
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="CostAnalysisReport")


@attr.s(auto_attribs=True)
class CostAnalysisReport:
    """
    Attributes:
        single_payrun (Union[Unset, PayRun]): This model is right at the very heart of the software.
            There is a PayRun for each period in which people are paid.
        total_pay (Union[Unset, float]):
        pension (Union[Unset, float]):
        aeo_fees (Union[Unset, float]):
        total_cost (Union[Unset, float]):
        employer_nic (Union[Unset, float]):
        net_payroll_cost (Union[Unset, float]):
        male_count (Union[Unset, int]):
        female_count (Union[Unset, int]):
        leaver_count (Union[Unset, int]):
        joiner_count (Union[Unset, int]):
        payment_after_leaving_count (Union[Unset, int]):
        recoverable_amounts (Union[Unset, RecoverableAmounts]):
        employer (Union[Unset, Item]):
        pay_period (Union[Unset, PayPeriods]):
        ordinal (Union[Unset, int]):
        period (Union[Unset, int]):
        period_to (Union[Unset, int]):
        start_period_name (Union[Unset, None, str]):
        end_period_name (Union[Unset, None, str]):
        start_date (Union[Unset, datetime.date]):
        end_date (Union[Unset, datetime.date]):
        report (Union[Unset, Report]):
        tax_year (Union[Unset, TaxYear]):
    """

    single_payrun: Union[Unset, PayRun] = UNSET
    total_pay: Union[Unset, float] = UNSET
    pension: Union[Unset, float] = UNSET
    aeo_fees: Union[Unset, float] = UNSET
    total_cost: Union[Unset, float] = UNSET
    employer_nic: Union[Unset, float] = UNSET
    net_payroll_cost: Union[Unset, float] = UNSET
    male_count: Union[Unset, int] = UNSET
    female_count: Union[Unset, int] = UNSET
    leaver_count: Union[Unset, int] = UNSET
    joiner_count: Union[Unset, int] = UNSET
    payment_after_leaving_count: Union[Unset, int] = UNSET
    recoverable_amounts: Union[Unset, RecoverableAmounts] = UNSET
    employer: Union[Unset, Item] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET
    period_to: Union[Unset, int] = UNSET
    start_period_name: Union[Unset, None, str] = UNSET
    end_period_name: Union[Unset, None, str] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    end_date: Union[Unset, datetime.date] = UNSET
    report: Union[Unset, Report] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        single_payrun: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.single_payrun, Unset):
            single_payrun = self.single_payrun.to_dict()

        total_pay = self.total_pay
        pension = self.pension
        aeo_fees = self.aeo_fees
        total_cost = self.total_cost
        employer_nic = self.employer_nic
        net_payroll_cost = self.net_payroll_cost
        male_count = self.male_count
        female_count = self.female_count
        leaver_count = self.leaver_count
        joiner_count = self.joiner_count
        payment_after_leaving_count = self.payment_after_leaving_count
        recoverable_amounts: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.recoverable_amounts, Unset):
            recoverable_amounts = self.recoverable_amounts.to_dict()

        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period
        period_to = self.period_to
        start_period_name = self.start_period_name
        end_period_name = self.end_period_name
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        report: Union[Unset, str] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.value

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if single_payrun is not UNSET:
            field_dict["singlePayrun"] = single_payrun
        if total_pay is not UNSET:
            field_dict["totalPay"] = total_pay
        if pension is not UNSET:
            field_dict["pension"] = pension
        if aeo_fees is not UNSET:
            field_dict["aeoFees"] = aeo_fees
        if total_cost is not UNSET:
            field_dict["totalCost"] = total_cost
        if employer_nic is not UNSET:
            field_dict["employerNic"] = employer_nic
        if net_payroll_cost is not UNSET:
            field_dict["netPayrollCost"] = net_payroll_cost
        if male_count is not UNSET:
            field_dict["maleCount"] = male_count
        if female_count is not UNSET:
            field_dict["femaleCount"] = female_count
        if leaver_count is not UNSET:
            field_dict["leaverCount"] = leaver_count
        if joiner_count is not UNSET:
            field_dict["joinerCount"] = joiner_count
        if payment_after_leaving_count is not UNSET:
            field_dict["paymentAfterLeavingCount"] = payment_after_leaving_count
        if recoverable_amounts is not UNSET:
            field_dict["recoverableAmounts"] = recoverable_amounts
        if employer is not UNSET:
            field_dict["employer"] = employer
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period
        if period_to is not UNSET:
            field_dict["periodTo"] = period_to
        if start_period_name is not UNSET:
            field_dict["startPeriodName"] = start_period_name
        if end_period_name is not UNSET:
            field_dict["endPeriodName"] = end_period_name
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if report is not UNSET:
            field_dict["report"] = report
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _single_payrun = d.pop("singlePayrun", UNSET)
        single_payrun: Union[Unset, PayRun]
        if isinstance(_single_payrun, Unset):
            single_payrun = UNSET
        else:
            single_payrun = PayRun.from_dict(_single_payrun)

        total_pay = d.pop("totalPay", UNSET)

        pension = d.pop("pension", UNSET)

        aeo_fees = d.pop("aeoFees", UNSET)

        total_cost = d.pop("totalCost", UNSET)

        employer_nic = d.pop("employerNic", UNSET)

        net_payroll_cost = d.pop("netPayrollCost", UNSET)

        male_count = d.pop("maleCount", UNSET)

        female_count = d.pop("femaleCount", UNSET)

        leaver_count = d.pop("leaverCount", UNSET)

        joiner_count = d.pop("joinerCount", UNSET)

        payment_after_leaving_count = d.pop("paymentAfterLeavingCount", UNSET)

        _recoverable_amounts = d.pop("recoverableAmounts", UNSET)
        recoverable_amounts: Union[Unset, RecoverableAmounts]
        if isinstance(_recoverable_amounts, Unset):
            recoverable_amounts = UNSET
        else:
            recoverable_amounts = RecoverableAmounts.from_dict(_recoverable_amounts)

        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, Item]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = Item.from_dict(_employer)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        period_to = d.pop("periodTo", UNSET)

        start_period_name = d.pop("startPeriodName", UNSET)

        end_period_name = d.pop("endPeriodName", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.date]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        _report = d.pop("report", UNSET)
        report: Union[Unset, Report]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = Report(_report)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        cost_analysis_report = cls(
            single_payrun=single_payrun,
            total_pay=total_pay,
            pension=pension,
            aeo_fees=aeo_fees,
            total_cost=total_cost,
            employer_nic=employer_nic,
            net_payroll_cost=net_payroll_cost,
            male_count=male_count,
            female_count=female_count,
            leaver_count=leaver_count,
            joiner_count=joiner_count,
            payment_after_leaving_count=payment_after_leaving_count,
            recoverable_amounts=recoverable_amounts,
            employer=employer,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
            period_to=period_to,
            start_period_name=start_period_name,
            end_period_name=end_period_name,
            start_date=start_date,
            end_date=end_date,
            report=report,
            tax_year=tax_year,
        )

        return cost_analysis_report



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PensionerPayroll")


@attr.s(auto_attribs=True)
class PensionerPayroll:
    """
    Attributes:
        in_receipt_of_pension (Union[Unset, bool]): If set to true then the FPS will have the OccPenInd flag set to
            'yes'
        bereaved (Union[Unset, bool]): Indicator that Occupational Pension is being paid because they are a recently
            bereaved Spouse/Civil Partner
        amount (Union[Unset, float]): Annual amount of occupational pension
    """

    in_receipt_of_pension: Union[Unset, bool] = UNSET
    bereaved: Union[Unset, bool] = UNSET
    amount: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        in_receipt_of_pension = self.in_receipt_of_pension
        bereaved = self.bereaved
        amount = self.amount

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if in_receipt_of_pension is not UNSET:
            field_dict["inReceiptOfPension"] = in_receipt_of_pension
        if bereaved is not UNSET:
            field_dict["bereaved"] = bereaved
        if amount is not UNSET:
            field_dict["amount"] = amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        in_receipt_of_pension = d.pop("inReceiptOfPension", UNSET)

        bereaved = d.pop("bereaved", UNSET)

        amount = d.pop("amount", UNSET)

        pensioner_payroll = cls(
            in_receipt_of_pension=in_receipt_of_pension,
            bereaved=bereaved,
            amount=amount,
        )

        return pensioner_payroll



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="LeaverDetails")


@attr.s(auto_attribs=True)
class LeaverDetails:
    """
    Attributes:
        has_left (Union[Unset, bool]):
        leave_date (Union[Unset, None, datetime.date]):
        is_deceased (Union[Unset, bool]):
        payment_after_leaving (Union[Unset, bool]):
        p_45_sent (Union[Unset, bool]):
    """

    has_left: Union[Unset, bool] = UNSET
    leave_date: Union[Unset, None, datetime.date] = UNSET
    is_deceased: Union[Unset, bool] = UNSET
    payment_after_leaving: Union[Unset, bool] = UNSET
    p_45_sent: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        has_left = self.has_left
        leave_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.leave_date, Unset):
            leave_date = self.leave_date.isoformat() if self.leave_date else None

        is_deceased = self.is_deceased
        payment_after_leaving = self.payment_after_leaving
        p_45_sent = self.p_45_sent

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if has_left is not UNSET:
            field_dict["hasLeft"] = has_left
        if leave_date is not UNSET:
            field_dict["leaveDate"] = leave_date
        if is_deceased is not UNSET:
            field_dict["isDeceased"] = is_deceased
        if payment_after_leaving is not UNSET:
            field_dict["paymentAfterLeaving"] = payment_after_leaving
        if p_45_sent is not UNSET:
            field_dict["p45Sent"] = p_45_sent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        has_left = d.pop("hasLeft", UNSET)

        _leave_date = d.pop("leaveDate", UNSET)
        leave_date: Union[Unset, None, datetime.date]
        if _leave_date is None:
            leave_date = None
        elif isinstance(_leave_date, Unset):
            leave_date = UNSET
        else:
            leave_date = isoparse(_leave_date).date()

        is_deceased = d.pop("isDeceased", UNSET)

        payment_after_leaving = d.pop("paymentAfterLeaving", UNSET)

        p_45_sent = d.pop("p45Sent", UNSET)

        leaver_details = cls(
            has_left=has_left,
            leave_date=leave_date,
            is_deceased=is_deceased,
            payment_after_leaving=payment_after_leaving,
            p_45_sent=p_45_sent,
        )

        return leaver_details



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pdf_paper_margins import PdfPaperMargins
from ..models.pdf_paper_orientation import PdfPaperOrientation
from ..models.pdf_paper_size import PdfPaperSize
from ..models.report_custom_css_option import ReportCustomCssOption
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayslipCustomisation")


@attr.s(auto_attribs=True)
class PayslipCustomisation:
    """Used to represent any customisations you make to the look of Payslip PDFs.
    This is covered in detail in the Guides section.

        Attributes:
            custom (Union[Unset, bool]): Whether or not you are customising the Payslip.
                This should always be true if you are providing any other values.
            include_logo (Union[Unset, bool]): Whether or not to include the employer logo on the payslip.
            custom_css_option (Union[Unset, ReportCustomCssOption]):
            custom_css (Union[Unset, None, str]):
            remove_ytd (Union[Unset, bool]): Whether or not to remove the YTD column
            include_employer_address (Union[Unset, bool]): Whether or not to include the employer address
            use_umbrella_format (Union[Unset, bool]): If true then the format the payslip will include details of the
                Umbrella Payment as well as employer costs
            filename (Union[Unset, None, str]):
            include_department (Union[Unset, bool]): Whether or not to include the primary department of the employee on the
                payslip
            include_health_and_social_care_message (Union[Unset, bool]): Whether or not to include health and social car
                message
            include_benefits (Union[Unset, bool]): Whether or not to include any payrolled benefits
            paper_size (Union[Unset, PdfPaperSize]):
            orientation (Union[Unset, PdfPaperOrientation]):
            margins (Union[Unset, PdfPaperMargins]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    custom: Union[Unset, bool] = UNSET
    include_logo: Union[Unset, bool] = UNSET
    custom_css_option: Union[Unset, ReportCustomCssOption] = UNSET
    custom_css: Union[Unset, None, str] = UNSET
    remove_ytd: Union[Unset, bool] = UNSET
    include_employer_address: Union[Unset, bool] = UNSET
    use_umbrella_format: Union[Unset, bool] = UNSET
    filename: Union[Unset, None, str] = UNSET
    include_department: Union[Unset, bool] = UNSET
    include_health_and_social_care_message: Union[Unset, bool] = UNSET
    include_benefits: Union[Unset, bool] = UNSET
    paper_size: Union[Unset, PdfPaperSize] = UNSET
    orientation: Union[Unset, PdfPaperOrientation] = UNSET
    margins: Union[Unset, PdfPaperMargins] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        custom = self.custom
        include_logo = self.include_logo
        custom_css_option: Union[Unset, str] = UNSET
        if not isinstance(self.custom_css_option, Unset):
            custom_css_option = self.custom_css_option.value

        custom_css = self.custom_css
        remove_ytd = self.remove_ytd
        include_employer_address = self.include_employer_address
        use_umbrella_format = self.use_umbrella_format
        filename = self.filename
        include_department = self.include_department
        include_health_and_social_care_message = (
            self.include_health_and_social_care_message
        )
        include_benefits = self.include_benefits
        paper_size: Union[Unset, str] = UNSET
        if not isinstance(self.paper_size, Unset):
            paper_size = self.paper_size.value

        orientation: Union[Unset, str] = UNSET
        if not isinstance(self.orientation, Unset):
            orientation = self.orientation.value

        margins: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.margins, Unset):
            margins = self.margins.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if custom is not UNSET:
            field_dict["custom"] = custom
        if include_logo is not UNSET:
            field_dict["includeLogo"] = include_logo
        if custom_css_option is not UNSET:
            field_dict["customCssOption"] = custom_css_option
        if custom_css is not UNSET:
            field_dict["customCss"] = custom_css
        if remove_ytd is not UNSET:
            field_dict["removeYtd"] = remove_ytd
        if include_employer_address is not UNSET:
            field_dict["includeEmployerAddress"] = include_employer_address
        if use_umbrella_format is not UNSET:
            field_dict["useUmbrellaFormat"] = use_umbrella_format
        if filename is not UNSET:
            field_dict["filename"] = filename
        if include_department is not UNSET:
            field_dict["includeDepartment"] = include_department
        if include_health_and_social_care_message is not UNSET:
            field_dict[
                "includeHealthAndSocialCareMessage"
            ] = include_health_and_social_care_message
        if include_benefits is not UNSET:
            field_dict["includeBenefits"] = include_benefits
        if paper_size is not UNSET:
            field_dict["paperSize"] = paper_size
        if orientation is not UNSET:
            field_dict["orientation"] = orientation
        if margins is not UNSET:
            field_dict["margins"] = margins
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        custom = d.pop("custom", UNSET)

        include_logo = d.pop("includeLogo", UNSET)

        _custom_css_option = d.pop("customCssOption", UNSET)
        custom_css_option: Union[Unset, ReportCustomCssOption]
        if isinstance(_custom_css_option, Unset):
            custom_css_option = UNSET
        else:
            custom_css_option = ReportCustomCssOption(_custom_css_option)

        custom_css = d.pop("customCss", UNSET)

        remove_ytd = d.pop("removeYtd", UNSET)

        include_employer_address = d.pop("includeEmployerAddress", UNSET)

        use_umbrella_format = d.pop("useUmbrellaFormat", UNSET)

        filename = d.pop("filename", UNSET)

        include_department = d.pop("includeDepartment", UNSET)

        include_health_and_social_care_message = d.pop(
            "includeHealthAndSocialCareMessage", UNSET
        )

        include_benefits = d.pop("includeBenefits", UNSET)

        _paper_size = d.pop("paperSize", UNSET)
        paper_size: Union[Unset, PdfPaperSize]
        if isinstance(_paper_size, Unset):
            paper_size = UNSET
        else:
            paper_size = PdfPaperSize(_paper_size)

        _orientation = d.pop("orientation", UNSET)
        orientation: Union[Unset, PdfPaperOrientation]
        if isinstance(_orientation, Unset):
            orientation = UNSET
        else:
            orientation = PdfPaperOrientation(_orientation)

        _margins = d.pop("margins", UNSET)
        margins: Union[Unset, PdfPaperMargins]
        if isinstance(_margins, Unset):
            margins = UNSET
        else:
            margins = PdfPaperMargins.from_dict(_margins)

        id = d.pop("id", UNSET)

        payslip_customisation = cls(
            custom=custom,
            include_logo=include_logo,
            custom_css_option=custom_css_option,
            custom_css=custom_css,
            remove_ytd=remove_ytd,
            include_employer_address=include_employer_address,
            use_umbrella_format=use_umbrella_format,
            filename=filename,
            include_department=include_department,
            include_health_and_social_care_message=include_health_and_social_care_message,
            include_benefits=include_benefits,
            paper_size=paper_size,
            orientation=orientation,
            margins=margins,
            id=id,
        )

        return payslip_customisation



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.emp_refs import EmpRefs
from ..models.fps_late_reason import FpsLateReason
from ..models.full_payment_submission import FullPaymentSubmission
from ..models.gov_talk_submission import GovTalkSubmission
from ..models.rti_validation_warning import RtiValidationWarning
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="Fps")


@attr.s(auto_attribs=True)
class Fps:
    """
    Attributes:
        late_reason (Union[Unset, FpsLateReason]):
        payment_date (Union[Unset, datetime.date]):
        employee_count (Union[Unset, int]):
        is_correction (Union[Unset, bool]):
        full_payment_submission (Union[Unset, FullPaymentSubmission]):
        validation_warnings (Union[Unset, None, List[RtiValidationWarning]]):
        i_rmark (Union[Unset, None, str]):
        xml (Union[Unset, None, str]):
        tax_year (Union[Unset, TaxYear]):
        employer_references (Union[Unset, EmpRefs]):
        gov_talk_submission (Union[Unset, GovTalkSubmission]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    late_reason: Union[Unset, FpsLateReason] = UNSET
    payment_date: Union[Unset, datetime.date] = UNSET
    employee_count: Union[Unset, int] = UNSET
    is_correction: Union[Unset, bool] = UNSET
    full_payment_submission: Union[Unset, FullPaymentSubmission] = UNSET
    validation_warnings: Union[Unset, None, List[RtiValidationWarning]] = UNSET
    i_rmark: Union[Unset, None, str] = UNSET
    xml: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    employer_references: Union[Unset, EmpRefs] = UNSET
    gov_talk_submission: Union[Unset, GovTalkSubmission] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        late_reason: Union[Unset, str] = UNSET
        if not isinstance(self.late_reason, Unset):
            late_reason = self.late_reason.value

        payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.payment_date, Unset):
            payment_date = self.payment_date.isoformat()

        employee_count = self.employee_count
        is_correction = self.is_correction
        full_payment_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.full_payment_submission, Unset):
            full_payment_submission = self.full_payment_submission.to_dict()

        validation_warnings: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.validation_warnings, Unset):
            if self.validation_warnings is None:
                validation_warnings = None
            else:
                validation_warnings = []
                for validation_warnings_item_data in self.validation_warnings:
                    validation_warnings_item = validation_warnings_item_data.to_dict()

                    validation_warnings.append(validation_warnings_item)

        i_rmark = self.i_rmark
        xml = self.xml
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        employer_references: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer_references, Unset):
            employer_references = self.employer_references.to_dict()

        gov_talk_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gov_talk_submission, Unset):
            gov_talk_submission = self.gov_talk_submission.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if late_reason is not UNSET:
            field_dict["lateReason"] = late_reason
        if payment_date is not UNSET:
            field_dict["paymentDate"] = payment_date
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if is_correction is not UNSET:
            field_dict["isCorrection"] = is_correction
        if full_payment_submission is not UNSET:
            field_dict["fullPaymentSubmission"] = full_payment_submission
        if validation_warnings is not UNSET:
            field_dict["validationWarnings"] = validation_warnings
        if i_rmark is not UNSET:
            field_dict["iRmark"] = i_rmark
        if xml is not UNSET:
            field_dict["xml"] = xml
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if employer_references is not UNSET:
            field_dict["employerReferences"] = employer_references
        if gov_talk_submission is not UNSET:
            field_dict["govTalkSubmission"] = gov_talk_submission
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _late_reason = d.pop("lateReason", UNSET)
        late_reason: Union[Unset, FpsLateReason]
        if isinstance(_late_reason, Unset):
            late_reason = UNSET
        else:
            late_reason = FpsLateReason(_late_reason)

        _payment_date = d.pop("paymentDate", UNSET)
        payment_date: Union[Unset, datetime.date]
        if isinstance(_payment_date, Unset):
            payment_date = UNSET
        else:
            payment_date = isoparse(_payment_date).date()

        employee_count = d.pop("employeeCount", UNSET)

        is_correction = d.pop("isCorrection", UNSET)

        _full_payment_submission = d.pop("fullPaymentSubmission", UNSET)
        full_payment_submission: Union[Unset, FullPaymentSubmission]
        if isinstance(_full_payment_submission, Unset):
            full_payment_submission = UNSET
        else:
            full_payment_submission = FullPaymentSubmission.from_dict(
                _full_payment_submission
            )

        validation_warnings = []
        _validation_warnings = d.pop("validationWarnings", UNSET)
        for validation_warnings_item_data in _validation_warnings or []:
            validation_warnings_item = RtiValidationWarning.from_dict(
                validation_warnings_item_data
            )

            validation_warnings.append(validation_warnings_item)

        i_rmark = d.pop("iRmark", UNSET)

        xml = d.pop("xml", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _employer_references = d.pop("employerReferences", UNSET)
        employer_references: Union[Unset, EmpRefs]
        if isinstance(_employer_references, Unset):
            employer_references = UNSET
        else:
            employer_references = EmpRefs.from_dict(_employer_references)

        _gov_talk_submission = d.pop("govTalkSubmission", UNSET)
        gov_talk_submission: Union[Unset, GovTalkSubmission]
        if isinstance(_gov_talk_submission, Unset):
            gov_talk_submission = UNSET
        else:
            gov_talk_submission = GovTalkSubmission.from_dict(_gov_talk_submission)

        id = d.pop("id", UNSET)

        fps = cls(
            late_reason=late_reason,
            payment_date=payment_date,
            employee_count=employee_count,
            is_correction=is_correction,
            full_payment_submission=full_payment_submission,
            validation_warnings=validation_warnings,
            i_rmark=i_rmark,
            xml=xml,
            tax_year=tax_year,
            employer_references=employer_references,
            gov_talk_submission=gov_talk_submission,
            id=id,
        )

        return fps



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="NationalInsuranceCode")


@attr.s(auto_attribs=True)
class NationalInsuranceCode:
    """Part of the TaxYearConfig that our engine uses to calculate National Insurance Contributions.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            code (Union[Unset, str]): [readonly] NI Table Letter
            description (Union[Unset, None, str]): [readonly] Description of Employees that would use this NI Letter
            ee_b (Union[Unset, float]): [readonly] Earnings at or above LEL up to and including PT (Employee Contribution)
            ee_c (Union[Unset, float]): [readonly] Earnings above the PT up to and including UEL (Employee Contribution)
            ee_d (Union[Unset, float]): [readonly] Balance of earnings above UEL (Employee Contribution)
            ee_e (Union[Unset, float]): [readonly]
            ee_f (Union[Unset, float]): [readonly]
            er_b (Union[Unset, float]): [readonly] Earnings at or above LEL up to and including PT (Employer Contribution)
            er_c (Union[Unset, float]): [readonly]  Earnings above the PT up to and including UEL (Employer Contribution)
            er_d (Union[Unset, float]): [readonly] Balance of earnings above UEL (Employer Contribution)
            er_e (Union[Unset, float]): [readonly]
            er_f (Union[Unset, float]): [readonly]
    """

    code: Union[Unset, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    ee_b: Union[Unset, float] = UNSET
    ee_c: Union[Unset, float] = UNSET
    ee_d: Union[Unset, float] = UNSET
    ee_e: Union[Unset, float] = UNSET
    ee_f: Union[Unset, float] = UNSET
    er_b: Union[Unset, float] = UNSET
    er_c: Union[Unset, float] = UNSET
    er_d: Union[Unset, float] = UNSET
    er_e: Union[Unset, float] = UNSET
    er_f: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        description = self.description
        ee_b = self.ee_b
        ee_c = self.ee_c
        ee_d = self.ee_d
        ee_e = self.ee_e
        ee_f = self.ee_f
        er_b = self.er_b
        er_c = self.er_c
        er_d = self.er_d
        er_e = self.er_e
        er_f = self.er_f

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if description is not UNSET:
            field_dict["description"] = description
        if ee_b is not UNSET:
            field_dict["eeB"] = ee_b
        if ee_c is not UNSET:
            field_dict["eeC"] = ee_c
        if ee_d is not UNSET:
            field_dict["eeD"] = ee_d
        if ee_e is not UNSET:
            field_dict["eeE"] = ee_e
        if ee_f is not UNSET:
            field_dict["eeF"] = ee_f
        if er_b is not UNSET:
            field_dict["erB"] = er_b
        if er_c is not UNSET:
            field_dict["erC"] = er_c
        if er_d is not UNSET:
            field_dict["erD"] = er_d
        if er_e is not UNSET:
            field_dict["erE"] = er_e
        if er_f is not UNSET:
            field_dict["erF"] = er_f

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        description = d.pop("description", UNSET)

        ee_b = d.pop("eeB", UNSET)

        ee_c = d.pop("eeC", UNSET)

        ee_d = d.pop("eeD", UNSET)

        ee_e = d.pop("eeE", UNSET)

        ee_f = d.pop("eeF", UNSET)

        er_b = d.pop("erB", UNSET)

        er_c = d.pop("erC", UNSET)

        er_d = d.pop("erD", UNSET)

        er_e = d.pop("erE", UNSET)

        er_f = d.pop("erF", UNSET)

        national_insurance_code = cls(
            code=code,
            description=description,
            ee_b=ee_b,
            ee_c=ee_c,
            ee_d=ee_d,
            ee_e=ee_e,
            ee_f=ee_f,
            er_b=er_b,
            er_c=er_c,
            er_d=er_d,
            er_e=er_e,
            er_f=er_f,
        )

        return national_insurance_code



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.p11d_asset_available import P11DAssetAvailable
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DAssetAvailableCollection")


@attr.s(auto_attribs=True)
class P11DAssetAvailableCollection:
    """
    Attributes:
        asset (Union[Unset, None, List[P11DAssetAvailable]]):
        type_letter (Union[Unset, None, str]):
    """

    asset: Union[Unset, None, List[P11DAssetAvailable]] = UNSET
    type_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        asset: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.asset, Unset):
            if self.asset is None:
                asset = None
            else:
                asset = []
                for asset_item_data in self.asset:
                    asset_item = asset_item_data.to_dict()

                    asset.append(asset_item)

        type_letter = self.type_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if asset is not UNSET:
            field_dict["asset"] = asset
        if type_letter is not UNSET:
            field_dict["typeLetter"] = type_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        asset = []
        _asset = d.pop("asset", UNSET)
        for asset_item_data in _asset or []:
            asset_item = P11DAssetAvailable.from_dict(asset_item_data)

            asset.append(asset_item)

        type_letter = d.pop("typeLetter", UNSET)

        p11d_asset_available_collection = cls(
            asset=asset,
            type_letter=type_letter,
        )

        return p11d_asset_available_collection



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="Loan")


@attr.s(auto_attribs=True)
class Loan:
    """Used to represent an Attachment of Earnings Order (AEO)

    Attributes:
        issue_date (Union[Unset, datetime.date]): The date the Loan was made
        reference (Union[Unset, None, str]): An optional reference for this Loan
        pay_code (Union[Unset, None, str]): The Code of the PayCode used for recording payments.
            The PayCode must have a CalculationType of FixedAmount and not be a multiplier code
        loan_amount (Union[Unset, float]): The initial amount that the loan was for
        period_amount (Union[Unset, float]): The amount to deduct per PayPeriod
        amount_repaid (Union[Unset, float]): [readonly] The amount repaid so far due to deductions in Payruns
        previously_paid (Union[Unset, float]): [readonly] Any amount by which to adjust the Balance, perhaps due to
            repayments made elsewhere
        balance (Union[Unset, float]): [readonly] The Balance of the Loan
        is_settled (Union[Unset, bool]): [readonly]
        is_paused (Union[Unset, bool]): If set to true then deductions will not be made
        document_count (Union[Unset, int]): [readonly] The number of attachments associated with this model
        documents (Union[Unset, None, List[Item]]): [readonly] The attachments associated with this model
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    issue_date: Union[Unset, datetime.date] = UNSET
    reference: Union[Unset, None, str] = UNSET
    pay_code: Union[Unset, None, str] = UNSET
    loan_amount: Union[Unset, float] = UNSET
    period_amount: Union[Unset, float] = UNSET
    amount_repaid: Union[Unset, float] = UNSET
    previously_paid: Union[Unset, float] = UNSET
    balance: Union[Unset, float] = UNSET
    is_settled: Union[Unset, bool] = UNSET
    is_paused: Union[Unset, bool] = UNSET
    document_count: Union[Unset, int] = UNSET
    documents: Union[Unset, None, List[Item]] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        issue_date: Union[Unset, str] = UNSET
        if not isinstance(self.issue_date, Unset):
            issue_date = self.issue_date.isoformat()

        reference = self.reference
        pay_code = self.pay_code
        loan_amount = self.loan_amount
        period_amount = self.period_amount
        amount_repaid = self.amount_repaid
        previously_paid = self.previously_paid
        balance = self.balance
        is_settled = self.is_settled
        is_paused = self.is_paused
        document_count = self.document_count
        documents: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.documents, Unset):
            if self.documents is None:
                documents = None
            else:
                documents = []
                for documents_item_data in self.documents:
                    documents_item = documents_item_data.to_dict()

                    documents.append(documents_item)

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if issue_date is not UNSET:
            field_dict["issueDate"] = issue_date
        if reference is not UNSET:
            field_dict["reference"] = reference
        if pay_code is not UNSET:
            field_dict["payCode"] = pay_code
        if loan_amount is not UNSET:
            field_dict["loanAmount"] = loan_amount
        if period_amount is not UNSET:
            field_dict["periodAmount"] = period_amount
        if amount_repaid is not UNSET:
            field_dict["amountRepaid"] = amount_repaid
        if previously_paid is not UNSET:
            field_dict["previouslyPaid"] = previously_paid
        if balance is not UNSET:
            field_dict["balance"] = balance
        if is_settled is not UNSET:
            field_dict["isSettled"] = is_settled
        if is_paused is not UNSET:
            field_dict["isPaused"] = is_paused
        if document_count is not UNSET:
            field_dict["documentCount"] = document_count
        if documents is not UNSET:
            field_dict["documents"] = documents
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _issue_date = d.pop("issueDate", UNSET)
        issue_date: Union[Unset, datetime.date]
        if isinstance(_issue_date, Unset):
            issue_date = UNSET
        else:
            issue_date = isoparse(_issue_date).date()

        reference = d.pop("reference", UNSET)

        pay_code = d.pop("payCode", UNSET)

        loan_amount = d.pop("loanAmount", UNSET)

        period_amount = d.pop("periodAmount", UNSET)

        amount_repaid = d.pop("amountRepaid", UNSET)

        previously_paid = d.pop("previouslyPaid", UNSET)

        balance = d.pop("balance", UNSET)

        is_settled = d.pop("isSettled", UNSET)

        is_paused = d.pop("isPaused", UNSET)

        document_count = d.pop("documentCount", UNSET)

        documents = []
        _documents = d.pop("documents", UNSET)
        for documents_item_data in _documents or []:
            documents_item = Item.from_dict(documents_item_data)

            documents.append(documents_item)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        loan = cls(
            issue_date=issue_date,
            reference=reference,
            pay_code=pay_code,
            loan_amount=loan_amount,
            period_amount=period_amount,
            amount_repaid=amount_repaid,
            previously_paid=previously_paid,
            balance=balance,
            is_settled=is_settled,
            is_paused=is_paused,
            document_count=document_count,
            documents=documents,
            employee=employee,
            id=id,
        )

        return loan



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.p11d_car import P11DCar
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DCarCollection")


@attr.s(auto_attribs=True)
class P11DCarCollection:
    """
    Attributes:
        car (Union[Unset, None, List[P11DCar]]):
        total_cars_or_relevant_amt (Union[Unset, None, str]):
        total_fuel_or_relevant_amt (Union[Unset, None, str]):
        type_letter (Union[Unset, None, str]):
    """

    car: Union[Unset, None, List[P11DCar]] = UNSET
    total_cars_or_relevant_amt: Union[Unset, None, str] = UNSET
    total_fuel_or_relevant_amt: Union[Unset, None, str] = UNSET
    type_letter: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        car: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.car, Unset):
            if self.car is None:
                car = None
            else:
                car = []
                for car_item_data in self.car:
                    car_item = car_item_data.to_dict()

                    car.append(car_item)

        total_cars_or_relevant_amt = self.total_cars_or_relevant_amt
        total_fuel_or_relevant_amt = self.total_fuel_or_relevant_amt
        type_letter = self.type_letter

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if car is not UNSET:
            field_dict["car"] = car
        if total_cars_or_relevant_amt is not UNSET:
            field_dict["totalCarsOrRelevantAmt"] = total_cars_or_relevant_amt
        if total_fuel_or_relevant_amt is not UNSET:
            field_dict["totalFuelOrRelevantAmt"] = total_fuel_or_relevant_amt
        if type_letter is not UNSET:
            field_dict["typeLetter"] = type_letter

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        car = []
        _car = d.pop("car", UNSET)
        for car_item_data in _car or []:
            car_item = P11DCar.from_dict(car_item_data)

            car.append(car_item)

        total_cars_or_relevant_amt = d.pop("totalCarsOrRelevantAmt", UNSET)

        total_fuel_or_relevant_amt = d.pop("totalFuelOrRelevantAmt", UNSET)

        type_letter = d.pop("typeLetter", UNSET)

        p11d_car_collection = cls(
            car=car,
            total_cars_or_relevant_amt=total_cars_or_relevant_amt,
            total_fuel_or_relevant_amt=total_fuel_or_relevant_amt,
            type_letter=type_letter,
        )

        return p11d_car_collection



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..models.note_type import NoteType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Note")


@attr.s(auto_attribs=True)
class Note:
    """Used to represent a Note for an Employee

    Attributes:
        note_date (Union[Unset, datetime.date]): The date of the note
        note_text (Union[Unset, None, str]): Note text
        created_by (Union[Unset, None, str]): [readonly] The email address of the user that create the Note
        updated_by (Union[Unset, None, str]): [readonly] The email address of the user that last updated the Note
        type (Union[Unset, NoteType]):
        document_count (Union[Unset, int]): [readonly] The number of attachments associated with this model
        documents (Union[Unset, None, List[Item]]): [readonly] The attachments associated with this model
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    note_date: Union[Unset, datetime.date] = UNSET
    note_text: Union[Unset, None, str] = UNSET
    created_by: Union[Unset, None, str] = UNSET
    updated_by: Union[Unset, None, str] = UNSET
    type: Union[Unset, NoteType] = UNSET
    document_count: Union[Unset, int] = UNSET
    documents: Union[Unset, None, List[Item]] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        note_date: Union[Unset, str] = UNSET
        if not isinstance(self.note_date, Unset):
            note_date = self.note_date.isoformat()

        note_text = self.note_text
        created_by = self.created_by
        updated_by = self.updated_by
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        document_count = self.document_count
        documents: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.documents, Unset):
            if self.documents is None:
                documents = None
            else:
                documents = []
                for documents_item_data in self.documents:
                    documents_item = documents_item_data.to_dict()

                    documents.append(documents_item)

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if note_date is not UNSET:
            field_dict["noteDate"] = note_date
        if note_text is not UNSET:
            field_dict["noteText"] = note_text
        if created_by is not UNSET:
            field_dict["createdBy"] = created_by
        if updated_by is not UNSET:
            field_dict["updatedBy"] = updated_by
        if type is not UNSET:
            field_dict["type"] = type
        if document_count is not UNSET:
            field_dict["documentCount"] = document_count
        if documents is not UNSET:
            field_dict["documents"] = documents
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _note_date = d.pop("noteDate", UNSET)
        note_date: Union[Unset, datetime.date]
        if isinstance(_note_date, Unset):
            note_date = UNSET
        else:
            note_date = isoparse(_note_date).date()

        note_text = d.pop("noteText", UNSET)

        created_by = d.pop("createdBy", UNSET)

        updated_by = d.pop("updatedBy", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, NoteType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = NoteType(_type)

        document_count = d.pop("documentCount", UNSET)

        documents = []
        _documents = d.pop("documents", UNSET)
        for documents_item_data in _documents or []:
            documents_item = Item.from_dict(documents_item_data)

            documents.append(documents_item)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        note = cls(
            note_date=note_date,
            note_text=note_text,
            created_by=created_by,
            updated_by=updated_by,
            type=type,
            document_count=document_count,
            documents=documents,
            employee=employee,
            id=id,
        )

        return note



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EpsDeMinimisStateAid")


@attr.s(auto_attribs=True)
class EpsDeMinimisStateAid:
    """Used on an EPS to declare an Employment Allowance DeMinimis State Aid information

    Attributes:
        agri (Union[Unset, bool]):
        fisheries_aqua (Union[Unset, bool]):
        road_trans (Union[Unset, bool]):
        indust (Union[Unset, bool]):
    """

    agri: Union[Unset, bool] = UNSET
    fisheries_aqua: Union[Unset, bool] = UNSET
    road_trans: Union[Unset, bool] = UNSET
    indust: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        agri = self.agri
        fisheries_aqua = self.fisheries_aqua
        road_trans = self.road_trans
        indust = self.indust

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if agri is not UNSET:
            field_dict["agri"] = agri
        if fisheries_aqua is not UNSET:
            field_dict["fisheriesAqua"] = fisheries_aqua
        if road_trans is not UNSET:
            field_dict["roadTrans"] = road_trans
        if indust is not UNSET:
            field_dict["indust"] = indust

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        agri = d.pop("agri", UNSET)

        fisheries_aqua = d.pop("fisheriesAqua", UNSET)

        road_trans = d.pop("roadTrans", UNSET)

        indust = d.pop("indust", UNSET)

        eps_de_minimis_state_aid = cls(
            agri=agri,
            fisheries_aqua=fisheries_aqua,
            road_trans=road_trans,
            indust=indust,
        )

        return eps_de_minimis_state_aid



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.background_task_status import BackgroundTaskStatus
from ..models.bank_details import BankDetails
from ..models.pay_run_payment import PayRunPayment
from ..types import UNSET, Unset

T = TypeVar("T", bound="BankPaymentInstruction")


@attr.s(auto_attribs=True)
class BankPaymentInstruction:
    """
    Attributes:
        originator (Union[Unset, BankDetails]):
        bank_payments (Union[Unset, None, List[PayRunPayment]]):
        status (Union[Unset, BackgroundTaskStatus]):
        status_message (Union[Unset, None, str]): [readonly] A message to elaborate on the Status
        link (Union[Unset, None, str]): [readonly] If available, a link to the payments in an ExternalDataProvider
        service_user_number (Union[Unset, None, str]): Service user number to be used while sending payment instruction
        bureau_number (Union[Unset, None, str]): Bureau number to be used if available while sending payment instruction
    """

    originator: Union[Unset, BankDetails] = UNSET
    bank_payments: Union[Unset, None, List[PayRunPayment]] = UNSET
    status: Union[Unset, BackgroundTaskStatus] = UNSET
    status_message: Union[Unset, None, str] = UNSET
    link: Union[Unset, None, str] = UNSET
    service_user_number: Union[Unset, None, str] = UNSET
    bureau_number: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        originator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.originator, Unset):
            originator = self.originator.to_dict()

        bank_payments: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bank_payments, Unset):
            if self.bank_payments is None:
                bank_payments = None
            else:
                bank_payments = []
                for bank_payments_item_data in self.bank_payments:
                    bank_payments_item = bank_payments_item_data.to_dict()

                    bank_payments.append(bank_payments_item)

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_message = self.status_message
        link = self.link
        service_user_number = self.service_user_number
        bureau_number = self.bureau_number

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if originator is not UNSET:
            field_dict["originator"] = originator
        if bank_payments is not UNSET:
            field_dict["bankPayments"] = bank_payments
        if status is not UNSET:
            field_dict["status"] = status
        if status_message is not UNSET:
            field_dict["statusMessage"] = status_message
        if link is not UNSET:
            field_dict["link"] = link
        if service_user_number is not UNSET:
            field_dict["serviceUserNumber"] = service_user_number
        if bureau_number is not UNSET:
            field_dict["bureauNumber"] = bureau_number

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _originator = d.pop("originator", UNSET)
        originator: Union[Unset, BankDetails]
        if isinstance(_originator, Unset):
            originator = UNSET
        else:
            originator = BankDetails.from_dict(_originator)

        bank_payments = []
        _bank_payments = d.pop("bankPayments", UNSET)
        for bank_payments_item_data in _bank_payments or []:
            bank_payments_item = PayRunPayment.from_dict(bank_payments_item_data)

            bank_payments.append(bank_payments_item)

        _status = d.pop("status", UNSET)
        status: Union[Unset, BackgroundTaskStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = BackgroundTaskStatus(_status)

        status_message = d.pop("statusMessage", UNSET)

        link = d.pop("link", UNSET)

        service_user_number = d.pop("serviceUserNumber", UNSET)

        bureau_number = d.pop("bureauNumber", UNSET)

        bank_payment_instruction = cls(
            originator=originator,
            bank_payments=bank_payments,
            status=status,
            status_message=status_message,
            link=link,
            service_user_number=service_user_number,
            bureau_number=bureau_number,
        )

        return bank_payment_instruction



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunChanges")


@attr.s(auto_attribs=True)
class PayRunChanges:
    """
    Attributes:
        pay_run_entries (Union[Unset, None, List[Item]]): A list of PayRunEntries where the PayRunEntry itself has been
            modified and.or the related Employee record was changed
    """

    pay_run_entries: Union[Unset, None, List[Item]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay_run_entries: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.pay_run_entries, Unset):
            if self.pay_run_entries is None:
                pay_run_entries = None
            else:
                pay_run_entries = []
                for pay_run_entries_item_data in self.pay_run_entries:
                    pay_run_entries_item = pay_run_entries_item_data.to_dict()

                    pay_run_entries.append(pay_run_entries_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay_run_entries is not UNSET:
            field_dict["payRunEntries"] = pay_run_entries

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pay_run_entries = []
        _pay_run_entries = d.pop("payRunEntries", UNSET)
        for pay_run_entries_item_data in _pay_run_entries or []:
            pay_run_entries_item = Item.from_dict(pay_run_entries_item_data)

            pay_run_entries.append(pay_run_entries_item)

        pay_run_changes = cls(
            pay_run_entries=pay_run_entries,
        )

        return pay_run_changes



from enum import Enum


class AuthScheme(str, Enum):
    OAUTH1 = "OAuth1"
    OAUTH2 = "OAuth2"
    BASIC = "Basic"
    APIKEYINHEADER = "ApiKeyInHeader"
    HMAC = "Hmac"
    OAUTH2CC = "OAuth2Cc"
    OAUTH2PASSWORD = "OAuth2Password"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.opening_balances import OpeningBalances
from ...types import Response


def _get_kwargs(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{employeeId}/openingBalances/terminationpayments".format(
        client.base_url, employerId=employer_id, employeeId=employee_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, OpeningBalances]]:
    if response.status_code == 200:
        response_200 = OpeningBalances.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, OpeningBalances]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Response[Union[Any, OpeningBalances]]:
    """Update Termination Pay

     Updates the Termination Pay value on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the Termination Pay value.
    Only the TerminationPayments property of the submitted OpeningBalances model is updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Optional[Union[Any, OpeningBalances]]:
    """Update Termination Pay

     Updates the Termination Pay value on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the Termination Pay value.
    Only the TerminationPayments property of the submitted OpeningBalances model is updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    return sync_detailed(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Response[Union[Any, OpeningBalances]]:
    """Update Termination Pay

     Updates the Termination Pay value on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the Termination Pay value.
    Only the TerminationPayments property of the submitted OpeningBalances model is updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        employee_id=employee_id,
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    employee_id: str,
    *,
    client: Client,
    json_body: OpeningBalances,
) -> Optional[Union[Any, OpeningBalances]]:
    """Update Termination Pay

     Updates the Termination Pay value on the Opening Balances for an Employee.
    This would ideally be done when the Opening Balances are updated but if payruns have already been
    started for the employee then Opening Balances can't be updated - hence this API call just to update
    the Termination Pay value.
    Only the TerminationPayments property of the submitted OpeningBalances model is updated.

    Args:
        employer_id (str):
        employee_id (str):
        json_body (OpeningBalances):

    Returns:
        Response[Union[Any, OpeningBalances]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            employee_id=employee_id,
            client=client,
            json_body=json_body,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.address import Address
from ..models.employment_details import EmploymentDetails
from ..models.hmrc_details import HmrcDetails
from ..models.item import Item
from ..models.pay_options import PayOptions
from ..models.pay_run_totals import PayRunTotals
from ..models.payslip_line import PayslipLine
from ..models.personal_details import PersonalDetails
from ..types import UNSET, Unset

T = TypeVar("T", bound="Payslip")


@attr.s(auto_attribs=True)
class Payslip:
    """If you don't want to use our customisable PDFs for Payslips then you can retrieve the raw data used to create a
    Payslip.
    This is the model you will be provided with for each employee.

        Attributes:
            is_closed (Union[Unset, bool]): [readonly] Indicates if the Payrun has been finalised
            period (Union[Unset, None, str]): [readonly] A description of the period that Payslip relates to.
            from_ (Union[Unset, datetime.date]): [readonly] The start date of the period this Payslip covers.
            to (Union[Unset, datetime.date]): [readonly] The end date of the period this Payslip covers.
            payment_date (Union[Unset, datetime.date]): [readonly] The date the Employee will be paid on
            note (Union[Unset, None, str]): [readonly] Any note that should appear on the payslip
            allowance_note (Union[Unset, None, str]): [readonly] Details of remaining allowance to show on payslip
            employee (Union[Unset, Item]):
            logo_url (Union[Unset, None, str]): [readonly] The Logo to include on the payslip
            employer (Union[Unset, Item]):
            employer_address (Union[Unset, Address]):
            hmrc_details (Union[Unset, HmrcDetails]):
            pay_options (Union[Unset, PayOptions]): This object forms the basis of the Employees payment.
            employment_details (Union[Unset, EmploymentDetails]):
            personal_details (Union[Unset, PersonalDetails]):
            totals (Union[Unset, PayRunTotals]): Used to represent totals for a PayRun or PayRunEntry.
                If a value is 0 then it will not be shown in the JSON.
            totals_ytd (Union[Unset, PayRunTotals]): Used to represent totals for a PayRun or PayRunEntry.
                If a value is 0 then it will not be shown in the JSON.
            lines (Union[Unset, None, List[PayslipLine]]): [readonly] The lines to display on the Payslip
            employer_ni (Union[Unset, float]): [readonly] The Employer NI Contribution amount
            employer_pension_contribs (Union[Unset, float]): [readonly] AThe Employer Pension Contribution Amount
    """

    is_closed: Union[Unset, bool] = UNSET
    period: Union[Unset, None, str] = UNSET
    from_: Union[Unset, datetime.date] = UNSET
    to: Union[Unset, datetime.date] = UNSET
    payment_date: Union[Unset, datetime.date] = UNSET
    note: Union[Unset, None, str] = UNSET
    allowance_note: Union[Unset, None, str] = UNSET
    employee: Union[Unset, Item] = UNSET
    logo_url: Union[Unset, None, str] = UNSET
    employer: Union[Unset, Item] = UNSET
    employer_address: Union[Unset, Address] = UNSET
    hmrc_details: Union[Unset, HmrcDetails] = UNSET
    pay_options: Union[Unset, PayOptions] = UNSET
    employment_details: Union[Unset, EmploymentDetails] = UNSET
    personal_details: Union[Unset, PersonalDetails] = UNSET
    totals: Union[Unset, PayRunTotals] = UNSET
    totals_ytd: Union[Unset, PayRunTotals] = UNSET
    lines: Union[Unset, None, List[PayslipLine]] = UNSET
    employer_ni: Union[Unset, float] = UNSET
    employer_pension_contribs: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_closed = self.is_closed
        period = self.period
        from_: Union[Unset, str] = UNSET
        if not isinstance(self.from_, Unset):
            from_ = self.from_.isoformat()

        to: Union[Unset, str] = UNSET
        if not isinstance(self.to, Unset):
            to = self.to.isoformat()

        payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.payment_date, Unset):
            payment_date = self.payment_date.isoformat()

        note = self.note
        allowance_note = self.allowance_note
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        logo_url = self.logo_url
        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        employer_address: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer_address, Unset):
            employer_address = self.employer_address.to_dict()

        hmrc_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hmrc_details, Unset):
            hmrc_details = self.hmrc_details.to_dict()

        pay_options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay_options, Unset):
            pay_options = self.pay_options.to_dict()

        employment_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employment_details, Unset):
            employment_details = self.employment_details.to_dict()

        personal_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.personal_details, Unset):
            personal_details = self.personal_details.to_dict()

        totals: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totals, Unset):
            totals = self.totals.to_dict()

        totals_ytd: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totals_ytd, Unset):
            totals_ytd = self.totals_ytd.to_dict()

        lines: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.lines, Unset):
            if self.lines is None:
                lines = None
            else:
                lines = []
                for lines_item_data in self.lines:
                    lines_item = lines_item_data.to_dict()

                    lines.append(lines_item)

        employer_ni = self.employer_ni
        employer_pension_contribs = self.employer_pension_contribs

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_closed is not UNSET:
            field_dict["isClosed"] = is_closed
        if period is not UNSET:
            field_dict["period"] = period
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to
        if payment_date is not UNSET:
            field_dict["paymentDate"] = payment_date
        if note is not UNSET:
            field_dict["note"] = note
        if allowance_note is not UNSET:
            field_dict["allowanceNote"] = allowance_note
        if employee is not UNSET:
            field_dict["employee"] = employee
        if logo_url is not UNSET:
            field_dict["logoUrl"] = logo_url
        if employer is not UNSET:
            field_dict["employer"] = employer
        if employer_address is not UNSET:
            field_dict["employerAddress"] = employer_address
        if hmrc_details is not UNSET:
            field_dict["hmrcDetails"] = hmrc_details
        if pay_options is not UNSET:
            field_dict["payOptions"] = pay_options
        if employment_details is not UNSET:
            field_dict["employmentDetails"] = employment_details
        if personal_details is not UNSET:
            field_dict["personalDetails"] = personal_details
        if totals is not UNSET:
            field_dict["totals"] = totals
        if totals_ytd is not UNSET:
            field_dict["totalsYtd"] = totals_ytd
        if lines is not UNSET:
            field_dict["lines"] = lines
        if employer_ni is not UNSET:
            field_dict["employerNi"] = employer_ni
        if employer_pension_contribs is not UNSET:
            field_dict["employerPensionContribs"] = employer_pension_contribs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_closed = d.pop("isClosed", UNSET)

        period = d.pop("period", UNSET)

        _from_ = d.pop("from", UNSET)
        from_: Union[Unset, datetime.date]
        if isinstance(_from_, Unset):
            from_ = UNSET
        else:
            from_ = isoparse(_from_).date()

        _to = d.pop("to", UNSET)
        to: Union[Unset, datetime.date]
        if isinstance(_to, Unset):
            to = UNSET
        else:
            to = isoparse(_to).date()

        _payment_date = d.pop("paymentDate", UNSET)
        payment_date: Union[Unset, datetime.date]
        if isinstance(_payment_date, Unset):
            payment_date = UNSET
        else:
            payment_date = isoparse(_payment_date).date()

        note = d.pop("note", UNSET)

        allowance_note = d.pop("allowanceNote", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        logo_url = d.pop("logoUrl", UNSET)

        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, Item]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = Item.from_dict(_employer)

        _employer_address = d.pop("employerAddress", UNSET)
        employer_address: Union[Unset, Address]
        if isinstance(_employer_address, Unset):
            employer_address = UNSET
        else:
            employer_address = Address.from_dict(_employer_address)

        _hmrc_details = d.pop("hmrcDetails", UNSET)
        hmrc_details: Union[Unset, HmrcDetails]
        if isinstance(_hmrc_details, Unset):
            hmrc_details = UNSET
        else:
            hmrc_details = HmrcDetails.from_dict(_hmrc_details)

        _pay_options = d.pop("payOptions", UNSET)
        pay_options: Union[Unset, PayOptions]
        if isinstance(_pay_options, Unset):
            pay_options = UNSET
        else:
            pay_options = PayOptions.from_dict(_pay_options)

        _employment_details = d.pop("employmentDetails", UNSET)
        employment_details: Union[Unset, EmploymentDetails]
        if isinstance(_employment_details, Unset):
            employment_details = UNSET
        else:
            employment_details = EmploymentDetails.from_dict(_employment_details)

        _personal_details = d.pop("personalDetails", UNSET)
        personal_details: Union[Unset, PersonalDetails]
        if isinstance(_personal_details, Unset):
            personal_details = UNSET
        else:
            personal_details = PersonalDetails.from_dict(_personal_details)

        _totals = d.pop("totals", UNSET)
        totals: Union[Unset, PayRunTotals]
        if isinstance(_totals, Unset):
            totals = UNSET
        else:
            totals = PayRunTotals.from_dict(_totals)

        _totals_ytd = d.pop("totalsYtd", UNSET)
        totals_ytd: Union[Unset, PayRunTotals]
        if isinstance(_totals_ytd, Unset):
            totals_ytd = UNSET
        else:
            totals_ytd = PayRunTotals.from_dict(_totals_ytd)

        lines = []
        _lines = d.pop("lines", UNSET)
        for lines_item_data in _lines or []:
            lines_item = PayslipLine.from_dict(lines_item_data)

            lines.append(lines_item)

        employer_ni = d.pop("employerNi", UNSET)

        employer_pension_contribs = d.pop("employerPensionContribs", UNSET)

        payslip = cls(
            is_closed=is_closed,
            period=period,
            from_=from_,
            to=to,
            payment_date=payment_date,
            note=note,
            allowance_note=allowance_note,
            employee=employee,
            logo_url=logo_url,
            employer=employer,
            employer_address=employer_address,
            hmrc_details=hmrc_details,
            pay_options=pay_options,
            employment_details=employment_details,
            personal_details=personal_details,
            totals=totals,
            totals_ytd=totals_ytd,
            lines=lines,
            employer_ni=employer_ni,
            employer_pension_contribs=employer_pension_contribs,
        )

        return payslip



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..models.pay_run_state import PayRunState
from ..models.pay_run_totals import PayRunTotals
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRun")


@attr.s(auto_attribs=True)
class PayRun:
    """This model is right at the very heart of the software.
    There is a PayRun for each period in which people are paid.

        Attributes:
            tax_year (Union[Unset, TaxYear]):
            tax_month (Union[Unset, int]): [readonly] The Tax Month that the Payment Date falls in
            pay_period (Union[Unset, PayPeriods]):
            ordinal (Union[Unset, int]): [readonly] Indicates whether this uses first, second, third (etc.) PaySchedule for
                this PayPeriod.
            period (Union[Unset, int]): [readonly] The period (i.e, Tax Week or Tax Month) that this PayRun is for.
            start_date (Union[Unset, datetime.date]): [readonly] The start date of the period this PayRun covers
            end_date (Union[Unset, datetime.date]): [readonly] The end date of the period this PayRun covers
            payment_date (Union[Unset, datetime.date]): [readonly] The intended date that Employees will be paid, although
                this can be changed on a per PayRunEntry basis
            employee_count (Union[Unset, int]): [readonly] The number of Employees included in this PayRun (including any
                CIS Subcontractors)
            sub_contractor_count (Union[Unset, int]): [readonly] The number of CIS Subcontractors included in this PayRun
            totals (Union[Unset, PayRunTotals]): Used to represent totals for a PayRun or PayRunEntry.
                If a value is 0 then it will not be shown in the JSON.
            state (Union[Unset, PayRunState]): The state of the payrun. You would set this value when updating a payrun to
                finalise or re-open it.
                Other states are used with Bureau functionality which isn't currently generally available.
            is_closed (Union[Unset, bool]): [readonly] Set to True if the PayRun is Finalised and changes can no longer be
                made
            date_closed (Union[Unset, None, datetime.datetime]):
            auto_pilot_close_date (Union[Unset, None, datetime.date]): [readonly] If AutoPilot is enabled in the
                AutomationSettings for the Employer then this property will tell you when
                the payrun will be automatically closed
            entries (Union[Unset, None, List[Item]]): [readonly] The PayRunEntries that make up this PayRun.
                This is populate automatically when you start a PayRun.
    """

    tax_year: Union[Unset, TaxYear] = UNSET
    tax_month: Union[Unset, int] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    ordinal: Union[Unset, int] = UNSET
    period: Union[Unset, int] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    end_date: Union[Unset, datetime.date] = UNSET
    payment_date: Union[Unset, datetime.date] = UNSET
    employee_count: Union[Unset, int] = UNSET
    sub_contractor_count: Union[Unset, int] = UNSET
    totals: Union[Unset, PayRunTotals] = UNSET
    state: Union[Unset, PayRunState] = UNSET
    is_closed: Union[Unset, bool] = UNSET
    date_closed: Union[Unset, None, datetime.datetime] = UNSET
    auto_pilot_close_date: Union[Unset, None, datetime.date] = UNSET
    entries: Union[Unset, None, List[Item]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        tax_month = self.tax_month
        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        ordinal = self.ordinal
        period = self.period
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.payment_date, Unset):
            payment_date = self.payment_date.isoformat()

        employee_count = self.employee_count
        sub_contractor_count = self.sub_contractor_count
        totals: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totals, Unset):
            totals = self.totals.to_dict()

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        is_closed = self.is_closed
        date_closed: Union[Unset, None, str] = UNSET
        if not isinstance(self.date_closed, Unset):
            date_closed = self.date_closed.isoformat() if self.date_closed else None

        auto_pilot_close_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.auto_pilot_close_date, Unset):
            auto_pilot_close_date = (
                self.auto_pilot_close_date.isoformat()
                if self.auto_pilot_close_date
                else None
            )

        entries: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.entries, Unset):
            if self.entries is None:
                entries = None
            else:
                entries = []
                for entries_item_data in self.entries:
                    entries_item = entries_item_data.to_dict()

                    entries.append(entries_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if tax_month is not UNSET:
            field_dict["taxMonth"] = tax_month
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if period is not UNSET:
            field_dict["period"] = period
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if payment_date is not UNSET:
            field_dict["paymentDate"] = payment_date
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if sub_contractor_count is not UNSET:
            field_dict["subContractorCount"] = sub_contractor_count
        if totals is not UNSET:
            field_dict["totals"] = totals
        if state is not UNSET:
            field_dict["state"] = state
        if is_closed is not UNSET:
            field_dict["isClosed"] = is_closed
        if date_closed is not UNSET:
            field_dict["dateClosed"] = date_closed
        if auto_pilot_close_date is not UNSET:
            field_dict["autoPilotCloseDate"] = auto_pilot_close_date
        if entries is not UNSET:
            field_dict["entries"] = entries

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        tax_month = d.pop("taxMonth", UNSET)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        ordinal = d.pop("ordinal", UNSET)

        period = d.pop("period", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.date]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        _payment_date = d.pop("paymentDate", UNSET)
        payment_date: Union[Unset, datetime.date]
        if isinstance(_payment_date, Unset):
            payment_date = UNSET
        else:
            payment_date = isoparse(_payment_date).date()

        employee_count = d.pop("employeeCount", UNSET)

        sub_contractor_count = d.pop("subContractorCount", UNSET)

        _totals = d.pop("totals", UNSET)
        totals: Union[Unset, PayRunTotals]
        if isinstance(_totals, Unset):
            totals = UNSET
        else:
            totals = PayRunTotals.from_dict(_totals)

        _state = d.pop("state", UNSET)
        state: Union[Unset, PayRunState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = PayRunState(_state)

        is_closed = d.pop("isClosed", UNSET)

        _date_closed = d.pop("dateClosed", UNSET)
        date_closed: Union[Unset, None, datetime.datetime]
        if _date_closed is None:
            date_closed = None
        elif isinstance(_date_closed, Unset):
            date_closed = UNSET
        else:
            date_closed = isoparse(_date_closed)

        _auto_pilot_close_date = d.pop("autoPilotCloseDate", UNSET)
        auto_pilot_close_date: Union[Unset, None, datetime.date]
        if _auto_pilot_close_date is None:
            auto_pilot_close_date = None
        elif isinstance(_auto_pilot_close_date, Unset):
            auto_pilot_close_date = UNSET
        else:
            auto_pilot_close_date = isoparse(_auto_pilot_close_date).date()

        entries = []
        _entries = d.pop("entries", UNSET)
        for entries_item_data in _entries or []:
            entries_item = Item.from_dict(entries_item_data)

            entries.append(entries_item)

        pay_run = cls(
            tax_year=tax_year,
            tax_month=tax_month,
            pay_period=pay_period,
            ordinal=ordinal,
            period=period,
            start_date=start_date,
            end_date=end_date,
            payment_date=payment_date,
            employee_count=employee_count,
            sub_contractor_count=sub_contractor_count,
            totals=totals,
            state=state,
            is_closed=is_closed,
            date_closed=date_closed,
            auto_pilot_close_date=auto_pilot_close_date,
            entries=entries,
        )

        return pay_run



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExbDeclarations")


@attr.s(auto_attribs=True)
class ExbDeclarations:
    """
    Attributes:
        p_11_dincluded (Union[Unset, None, str]):
        p_46_car_declaration (Union[Unset, None, str]):
    """

    p_11_dincluded: Union[Unset, None, str] = UNSET
    p_46_car_declaration: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        p_11_dincluded = self.p_11_dincluded
        p_46_car_declaration = self.p_46_car_declaration

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if p_11_dincluded is not UNSET:
            field_dict["p11Dincluded"] = p_11_dincluded
        if p_46_car_declaration is not UNSET:
            field_dict["p46CarDeclaration"] = p_46_car_declaration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        p_11_dincluded = d.pop("p11Dincluded", UNSET)

        p_46_car_declaration = d.pop("p46CarDeclaration", UNSET)

        exb_declarations = cls(
            p_11_dincluded=p_11_dincluded,
            p_46_car_declaration=p_46_car_declaration,
        )

        return exb_declarations



from typing import Any, Dict, Union

import httpx

from ...client import Client
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
) -> Dict[str, Any]:
    url = "{}/engine/config".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_tax_year: Union[Unset, None, str] = UNSET
    if not isinstance(tax_year, Unset):
        json_tax_year = tax_year.value if tax_year else None

    params["taxYear"] = json_tax_year

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
) -> Response[Any]:
    """Get Configuration

     Returns the raw configuration data that is used as the basis for any calculations for the given tax
    year.
    You would never need to use this configuration values yourself in any API calls. It is provided just
    for information purposes

    Args:
        tax_year (Union[Unset, None, TaxYear]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        tax_year=tax_year,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
) -> Response[Any]:
    """Get Configuration

     Returns the raw configuration data that is used as the basis for any calculations for the given tax
    year.
    You would never need to use this configuration values yourself in any API calls. It is provided just
    for information purposes

    Args:
        tax_year (Union[Unset, None, TaxYear]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        tax_year=tax_year,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from enum import Enum


class NoteType(str, Enum):
    GENERAL = "General"
    NEWSTARTERSTATEMENT = "NewStarterStatement"
    RTWPROOF = "RtwProof"
    P45 = "P45"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.item import Item
from ...models.pay_periods import PayPeriods
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{taxYear}/{payPeriod}".format(
        client.base_url, employerId=employer_id, taxYear=tax_year, payPeriod=pay_period
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[Item]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[Item]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[List[Item]]:
    """List PayRuns

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        client=client,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[List[Item]]:
    """List PayRuns

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        client=client,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[List[Item]]:
    """List PayRuns

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        client=client,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[List[Item]]:
    """List PayRuns

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            pay_period=pay_period,
            client=client,
            ordinal=ordinal,
        )
    ).parsed



import datetime
from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.calendar_entry import CalendarEntry
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: str,
    *,
    client: Client,
    from_: Union[Unset, None, datetime.datetime] = UNSET,
    to: Union[Unset, None, datetime.datetime] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employers/{id}/calendar".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_from_: Union[Unset, None, str] = UNSET
    if not isinstance(from_, Unset):
        json_from_ = from_.isoformat() if from_ else None

    params["from"] = json_from_

    json_to: Union[Unset, None, str] = UNSET
    if not isinstance(to, Unset):
        json_to = to.isoformat() if to else None

    params["to"] = json_to

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[CalendarEntry]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = CalendarEntry.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[CalendarEntry]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Client,
    from_: Union[Unset, None, datetime.datetime] = UNSET,
    to: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[List[CalendarEntry]]:
    """Get Employer Calendar

     Get a list of upcoming CalendarEntry for the Employer.

    Args:
        id (str):
        from_ (Union[Unset, None, datetime.datetime]):
        to (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[List[CalendarEntry]]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        from_=from_,
        to=to,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    *,
    client: Client,
    from_: Union[Unset, None, datetime.datetime] = UNSET,
    to: Union[Unset, None, datetime.datetime] = UNSET,
) -> Optional[List[CalendarEntry]]:
    """Get Employer Calendar

     Get a list of upcoming CalendarEntry for the Employer.

    Args:
        id (str):
        from_ (Union[Unset, None, datetime.datetime]):
        to (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[List[CalendarEntry]]
    """

    return sync_detailed(
        id=id,
        client=client,
        from_=from_,
        to=to,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Client,
    from_: Union[Unset, None, datetime.datetime] = UNSET,
    to: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[List[CalendarEntry]]:
    """Get Employer Calendar

     Get a list of upcoming CalendarEntry for the Employer.

    Args:
        id (str):
        from_ (Union[Unset, None, datetime.datetime]):
        to (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[List[CalendarEntry]]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        from_=from_,
        to=to,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    *,
    client: Client,
    from_: Union[Unset, None, datetime.datetime] = UNSET,
    to: Union[Unset, None, datetime.datetime] = UNSET,
) -> Optional[List[CalendarEntry]]:
    """Get Employer Calendar

     Get a list of upcoming CalendarEntry for the Employer.

    Args:
        id (str):
        from_ (Union[Unset, None, datetime.datetime]):
        to (Union[Unset, None, datetime.datetime]):

    Returns:
        Response[List[CalendarEntry]]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            from_=from_,
            to=to,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="PdfPaperMargins")


@attr.s(auto_attribs=True)
class PdfPaperMargins:
    """
    Attributes:
        top (Union[Unset, float]):
        right (Union[Unset, float]):
        bottom (Union[Unset, float]):
        left (Union[Unset, float]):
    """

    top: Union[Unset, float] = UNSET
    right: Union[Unset, float] = UNSET
    bottom: Union[Unset, float] = UNSET
    left: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        top = self.top
        right = self.right
        bottom = self.bottom
        left = self.left

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if top is not UNSET:
            field_dict["top"] = top
        if right is not UNSET:
            field_dict["right"] = right
        if bottom is not UNSET:
            field_dict["bottom"] = bottom
        if left is not UNSET:
            field_dict["left"] = left

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        top = d.pop("top", UNSET)

        right = d.pop("right", UNSET)

        bottom = d.pop("bottom", UNSET)

        left = d.pop("left", UNSET)

        pdf_paper_margins = cls(
            top=top,
            right=right,
            bottom=bottom,
            left=left,
        )

        return pdf_paper_margins



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.ae_action import AeAction
from ..models.ae_employee_state import AeEmployeeState
from ..models.ae_statutory_letter import AeStatutoryLetter
from ..types import UNSET, Unset

T = TypeVar("T", bound="AeAssessmentAction")


@attr.s(auto_attribs=True)
class AeAssessmentAction:
    """
    Attributes:
        action (Union[Unset, AeAction]):
        employee_state (Union[Unset, AeEmployeeState]):
        action_completed (Union[Unset, bool]): [readonly] Indicates whether or not the required action was successfully
            completed
        action_completed_message (Union[Unset, None, str]): [readonly] Gives further information about the action taken
            or the reason if wasn't successfully completed
        required_letter (Union[Unset, AeStatutoryLetter]):
        pension_scheme_id (Union[Unset, str]): [readonly] The PensionSchemeId that a completed action relates to
        worker_group_id (Union[Unset, str]): [readonly] The WorkerGroupId that a completed action relates to
        letter_not_yet_sent (Union[Unset, bool]): [readonly] Indicates whether or not any required letter has been sent
    """

    action: Union[Unset, AeAction] = UNSET
    employee_state: Union[Unset, AeEmployeeState] = UNSET
    action_completed: Union[Unset, bool] = UNSET
    action_completed_message: Union[Unset, None, str] = UNSET
    required_letter: Union[Unset, AeStatutoryLetter] = UNSET
    pension_scheme_id: Union[Unset, str] = UNSET
    worker_group_id: Union[Unset, str] = UNSET
    letter_not_yet_sent: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        action: Union[Unset, str] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.value

        employee_state: Union[Unset, str] = UNSET
        if not isinstance(self.employee_state, Unset):
            employee_state = self.employee_state.value

        action_completed = self.action_completed
        action_completed_message = self.action_completed_message
        required_letter: Union[Unset, str] = UNSET
        if not isinstance(self.required_letter, Unset):
            required_letter = self.required_letter.value

        pension_scheme_id = self.pension_scheme_id
        worker_group_id = self.worker_group_id
        letter_not_yet_sent = self.letter_not_yet_sent

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if action is not UNSET:
            field_dict["action"] = action
        if employee_state is not UNSET:
            field_dict["employeeState"] = employee_state
        if action_completed is not UNSET:
            field_dict["actionCompleted"] = action_completed
        if action_completed_message is not UNSET:
            field_dict["actionCompletedMessage"] = action_completed_message
        if required_letter is not UNSET:
            field_dict["requiredLetter"] = required_letter
        if pension_scheme_id is not UNSET:
            field_dict["pensionSchemeId"] = pension_scheme_id
        if worker_group_id is not UNSET:
            field_dict["workerGroupId"] = worker_group_id
        if letter_not_yet_sent is not UNSET:
            field_dict["letterNotYetSent"] = letter_not_yet_sent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _action = d.pop("action", UNSET)
        action: Union[Unset, AeAction]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = AeAction(_action)

        _employee_state = d.pop("employeeState", UNSET)
        employee_state: Union[Unset, AeEmployeeState]
        if isinstance(_employee_state, Unset):
            employee_state = UNSET
        else:
            employee_state = AeEmployeeState(_employee_state)

        action_completed = d.pop("actionCompleted", UNSET)

        action_completed_message = d.pop("actionCompletedMessage", UNSET)

        _required_letter = d.pop("requiredLetter", UNSET)
        required_letter: Union[Unset, AeStatutoryLetter]
        if isinstance(_required_letter, Unset):
            required_letter = UNSET
        else:
            required_letter = AeStatutoryLetter(_required_letter)

        pension_scheme_id = d.pop("pensionSchemeId", UNSET)

        worker_group_id = d.pop("workerGroupId", UNSET)

        letter_not_yet_sent = d.pop("letterNotYetSent", UNSET)

        ae_assessment_action = cls(
            action=action,
            employee_state=employee_state,
            action_completed=action_completed,
            action_completed_message=action_completed_message,
            required_letter=required_letter,
            pension_scheme_id=pension_scheme_id,
            worker_group_id=worker_group_id,
            letter_not_yet_sent=letter_not_yet_sent,
        )

        return ae_assessment_action



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BankDetails")


@attr.s(auto_attribs=True)
class BankDetails:
    """
    Attributes:
        bank_name (Union[Unset, None, str]):
        bank_branch (Union[Unset, None, str]):
        bank_reference (Union[Unset, None, str]):
        account_name (Union[Unset, None, str]):
        account_number (Union[Unset, None, str]):
        sort_code (Union[Unset, None, str]):
        note (Union[Unset, None, str]):
    """

    bank_name: Union[Unset, None, str] = UNSET
    bank_branch: Union[Unset, None, str] = UNSET
    bank_reference: Union[Unset, None, str] = UNSET
    account_name: Union[Unset, None, str] = UNSET
    account_number: Union[Unset, None, str] = UNSET
    sort_code: Union[Unset, None, str] = UNSET
    note: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        bank_name = self.bank_name
        bank_branch = self.bank_branch
        bank_reference = self.bank_reference
        account_name = self.account_name
        account_number = self.account_number
        sort_code = self.sort_code
        note = self.note

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if bank_name is not UNSET:
            field_dict["bankName"] = bank_name
        if bank_branch is not UNSET:
            field_dict["bankBranch"] = bank_branch
        if bank_reference is not UNSET:
            field_dict["bankReference"] = bank_reference
        if account_name is not UNSET:
            field_dict["accountName"] = account_name
        if account_number is not UNSET:
            field_dict["accountNumber"] = account_number
        if sort_code is not UNSET:
            field_dict["sortCode"] = sort_code
        if note is not UNSET:
            field_dict["note"] = note

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bank_name = d.pop("bankName", UNSET)

        bank_branch = d.pop("bankBranch", UNSET)

        bank_reference = d.pop("bankReference", UNSET)

        account_name = d.pop("accountName", UNSET)

        account_number = d.pop("accountNumber", UNSET)

        sort_code = d.pop("sortCode", UNSET)

        note = d.pop("note", UNSET)

        bank_details = cls(
            bank_name=bank_name,
            bank_branch=bank_branch,
            bank_reference=bank_reference,
            account_name=account_name,
            account_number=account_number,
            sort_code=sort_code,
            note=note,
        )

        return bank_details



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..models.ytd_value import YtdValue
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployeeYtdValues")


@attr.s(auto_attribs=True)
class EmployeeYtdValues:
    """
    Attributes:
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        first_name (Union[Unset, None, str]):
        last_name (Union[Unset, None, str]):
        values (Union[Unset, None, List[YtdValue]]):
    """

    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    first_name: Union[Unset, None, str] = UNSET
    last_name: Union[Unset, None, str] = UNSET
    values: Union[Unset, None, List[YtdValue]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        first_name = self.first_name
        last_name = self.last_name
        values: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.values, Unset):
            if self.values is None:
                values = None
            else:
                values = []
                for values_item_data in self.values:
                    values_item = values_item_data.to_dict()

                    values.append(values_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        values = []
        _values = d.pop("values", UNSET)
        for values_item_data in _values or []:
            values_item = YtdValue.from_dict(values_item_data)

            values.append(values_item)

        employee_ytd_values = cls(
            employee=employee,
            payroll_code=payroll_code,
            first_name=first_name,
            last_name=last_name,
            values=values,
        )

        return employee_ytd_values



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RtiEmployeeAddress")


@attr.s(auto_attribs=True)
class RtiEmployeeAddress:
    """
    Attributes:
        line (Union[Unset, None, List[str]]):
        postcode (Union[Unset, None, str]):
        post_code (Union[Unset, None, str]):
        uk_postcode (Union[Unset, None, str]):
        country (Union[Unset, None, str]):
        foreign_country (Union[Unset, None, str]):
    """

    line: Union[Unset, None, List[str]] = UNSET
    postcode: Union[Unset, None, str] = UNSET
    post_code: Union[Unset, None, str] = UNSET
    uk_postcode: Union[Unset, None, str] = UNSET
    country: Union[Unset, None, str] = UNSET
    foreign_country: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        line: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.line, Unset):
            if self.line is None:
                line = None
            else:
                line = self.line

        postcode = self.postcode
        post_code = self.post_code
        uk_postcode = self.uk_postcode
        country = self.country
        foreign_country = self.foreign_country

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if line is not UNSET:
            field_dict["line"] = line
        if postcode is not UNSET:
            field_dict["postcode"] = postcode
        if post_code is not UNSET:
            field_dict["postCode"] = post_code
        if uk_postcode is not UNSET:
            field_dict["ukPostcode"] = uk_postcode
        if country is not UNSET:
            field_dict["country"] = country
        if foreign_country is not UNSET:
            field_dict["foreignCountry"] = foreign_country

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        line = cast(List[str], d.pop("line", UNSET))

        postcode = d.pop("postcode", UNSET)

        post_code = d.pop("postCode", UNSET)

        uk_postcode = d.pop("ukPostcode", UNSET)

        country = d.pop("country", UNSET)

        foreign_country = d.pop("foreignCountry", UNSET)

        rti_employee_address = cls(
            line=line,
            postcode=postcode,
            post_code=post_code,
            uk_postcode=uk_postcode,
            country=country,
            foreign_country=foreign_country,
        )

        return rti_employee_address
